@@ -1,15 +1,13 @@
 #
 #    FormValidator.pm - Object that validates form input data.
 #
-#    This file is part of Data::FormValidator.
+#    This file is part of FormValidator.
 #
 #    Author: Francis J. Lacoste <francis.lacoste@iNsu.COM>
-#    Maintainer: Mark Stosberg <mark@stosberg.com>
 #
 #    Copyright (C) 1999 Francis J. Lacoste, iNsu Innovations
 #    Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
 #    Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
-#    Parts Copyright 2001-2002 by Mark Stosberg <mark@stosberg.com>
 #
 #    Parts of this module are based on work by
 #    Bruce Albrecht, <bruce.albrecht@seag.fingerhut.com> contributed to
@@ -17,85 +15,86 @@
 #
 #    Parts also based on work by Michael J. Heins <mikeh@minivend.com>
 #
+#    Numerous changes by Mark Stosberg <mark@summersault.com>
+#
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms same terms as perl itself.
 #
-#    $Header: /cvsroot/cascade/dfv/lib/Data/FormValidator.pm,v 1.3 2001/11/03 18:09:46 markjugg Exp $
 package Data::FormValidator;
 
 use vars qw( $VERSION );
 
-$VERSION = '1.11';
+BEGIN {
+    ($VERSION) = '$Revision: 1.3 $' =~ /Revision: ([\d.]+)/;
+}
 
 require Exporter;
 @ISA = qw(Exporter);
 
 @EXPORT_OK = qw(
-       	filter_alphanum
-	filter_decimal
-	filter_digit
-	filter_integer
-	filter_lc
-	filter_neg_decimal
-	filter_neg_integer
-	filter_phone
-	filter_pos_decimal
-	filter_pos_integer
-	filter_quotemeta
-	filter_sql_wildcard
-	filter_strip
-	filter_trim
-	filter_uc
-	filter_ucfirst
-	valid_american_phone
-	valid_cc_exp
-	valid_cc_number
-	valid_cc_type
+          filter_trim
+          filter_strip
+          filter_digit
+          filter_alphanum
+          filter_integer
+          filter_pos_integer
+          filter_neg_integer
+          filter_decimal
+          filter_pos_decimal
+          filter_neg_decimal
+          filter_dollars
+          filter_phone
+          filter_sql_wildcard
+          filter_quotemeta
+          filter_lc
+          filter_uc
+          filter_ucfirst
 	valid_email
-	valid_ip_address
-	valid_phone
-	valid_postcode
-	valid_province
-	valid_state
 	valid_state_or_province
-	valid_zip
+	valid_state
+	valid_province
 	valid_zip_or_postcode
+	valid_postcode
+	valid_zip
+	valid_phone
+	valid_american_phone
+	valid_cc_number
+	valid_cc_exp
+	valid_cc_type
 );
 
 %EXPORT_TAGS = (
     filters => [qw/
-          filter_alphanum
-          filter_decimal
-          filter_digit
-          filter_dollars
-          filter_integer
-          filter_lc
-          filter_neg_decimal
-          filter_neg_integer
-          filter_phone
-          filter_pos_decimal
-          filter_pos_integer
-          filter_quotemeta
-          filter_sql_wildcard
-          filter_strip
           filter_trim
+          filter_strip
+          filter_digit
+          filter_alphanum
+          filter_integer
+          filter_pos_integer
+          filter_neg_integer
+          filter_decimal
+          filter_pos_decimal
+          filter_neg_decimal
+          filter_dollars
+          filter_phone
+          filter_sql_wildcard
+          filter_quotemeta
+          filter_lc
           filter_uc
-          filter_ucfirst
-    /],
+          filter_ucfirst/],
     validators => [qw/
-	valid_american_phone
-	valid_cc_exp
-	valid_cc_number
-	valid_cc_type
 	valid_email
-	valid_ip_address
-	valid_phone
-	valid_postcode
-	valid_province
-	valid_state
 	valid_state_or_province
-	valid_zip
+	valid_state
+	valid_province
 	valid_zip_or_postcode
+	valid_postcode
+	valid_zip
+	valid_phone
+	valid_american_phone
+	valid_cc_number
+	valid_cc_exp
+	valid_cc_type
 /],
 );
 
@@ -110,14 +109,10 @@ on input profile.
 
 =head1 SYNOPSIS
 
+In an HTML::Empberl page:
+
     use Data::FormValidator;
 
-	# For the common case of a validating a single profile provided through a hash reference,
-	# using 'validate' like this is the simplest solution 
-	my ($valids, $missings, $invalids, $unknowns) 
-		= Data::FormValidator->validate(\%fdat, \%profile);
-
-    # This is an example of using a validation profile defined in a seperate file
     my $validator = new Data::FormValidator( "/home/user/input_profiles.pl" );
     my ( $valid, $missing, $invalid, $unknown ) = $validator->validate(  \%fdat, "customer_infos" );
 
@@ -161,8 +156,7 @@ sub new {
 	$profile_file	= undef;
     }
 
-    bless { 
-		profile_file => $profile_file,
+    bless { profile_file => $profile_file,
 	    profiles	 => $profiles,
 	  }, $class;
 }
@@ -217,10 +211,7 @@ Here is an example of a valid input profiles specification :
 	customer_billing_infos => {
 	     optional	    => [ "cc_no" ],
 	     dependencies   => {
-            "cc_no" => [ qw( cc_type cc_exp ) ],
-            "pay_type" => {
-                check => [ qw( check_no ) ],
-             }
+		"cc_no" => [ qw( cc_type cc_exp ) ],
 	     },
 	     constraints => {
 		cc_no      => {  constraint  => "cc_number",
@@ -231,9 +222,6 @@ Here is an example of a valid input profiles specification :
 	      }
 	    filters       => [ "trim" ],
 	    field_filters => { cc_no => "digit" },
-	    field_filter_regexp_map => {
-			'/_name$/'	=> 'ucfirst',
-	    },
 	},
     }
 
@@ -280,14 +268,10 @@ to be optional, you could use the regular expression, /^user_/
 
 This is an hash reference which contains dependencies information.
 This is for the case where one optional fields has other requirements.
-The dependencies can be specified with an array reference.  For example,
-if you enter your credit card number, the field cc_exp and cc_type should
-also be present.  If the dependencies are specified with a hash reference
-then the additional constraint is added that the optional field must equal
-a key for the dependencies to be added. For example, if the pay_type field
-is equal to "check" then the check_no field is required.  Any fields in
-the dependencies list that is missing when the target is present will be
-reported as missing.
+For example, if you enter your credit card number, the field cc_exp
+and cc_type should also be present. Any fields in the dependencies
+list that is missing when the target is present will be reported as
+missing.
 
 =item dependency_groups
 
@@ -318,16 +302,6 @@ filters which will be applied to specific input fields. The key of the
 hash is the name of the input field and the value is a reference to an
 array of filters, the same way the filters parameter works.
 
-=item field_filter_regexp_map
-
-This is a hash reference where the keys are the regular expressions to
-use and the values are references to arrays of filters which will be
-applied to specific input fields. Used to apply filters to fields that
-match a regular expression. For example, you could make the first letter
-uppercase of all fields that end in "_name" by using the key '/_name$/'
-and the value "ucfirst".
-
-
 =item constraints
 
 This is a reference to an hash which contains the constraints that
@@ -354,12 +328,6 @@ you could check to see that all fields that end in "_postcode" are
 valid Canadian postal codes by using the key '_postcode$' and the
 value "postcode".
 
-=item missing_optional_valid
-
-This can be set to a true value (such as 1) to cause missing optional
-fields to be included in the valid hash. By default they are not
-included-- this is the historical behavior. 
-
 =back
 
 =cut
@@ -443,16 +411,14 @@ sub validate {
     my ( $self, $data, $name ) = @_;
 
     my $profile;
-	if ( ref $name ) {
-		$profile = $name;
-	} else {
-		$self->load_profiles;
-		$profile = $self->{profiles}{$name};
-		die "No such profile $name\n" unless $profile;
-	}
-
-	# check the profile syntax or die with an error. 
-	_check_profile_syntax($profile);
+    if ( ref $name ) {
+	$profile = $name;
+    } else {
+	$self->load_profiles;
+	$profile = $self->{profiles}{$name};
+	die "No such profile $name\n" unless $profile;
+    }
+    die "Invalid input profile\n" unless ref $profile eq "HASH";
 
     # Copy data and assumes that all is valid
     my %valid	    = %$data;
@@ -485,24 +451,6 @@ sub validate {
 		}
     }   
 
-	# add in specific filters from the regexp map
-	while ( my ($re,$filters) = each %{$profile->{field_filter_regexp_map} }) {
-		my $sub = eval 'sub { $_[0] =~ '. $re . '}';
-		die "Error compiling regular expression $re: $@" if $@;
-
-		foreach my $filter ( _arrayify($filters)) {
-			if (defined $filter) {
-				# Qualify symbolic references
-				$filter = ref $filter ? $filter : "filter_" . $filter;
-				no strict 'refs';
-
-				# find all the keys that match this RE and apply filters to them
-				map { $valid{$_} = $filter->( $valid{$_} ) }
-					grep { $sub->($_) } (keys %valid);
-			}	
-		}
-	}
- 
     my %required    = map { $_ => 1 } _arrayify($profile->{required});
     my %optional    = map { $_ => 1 } _arrayify($profile->{optional});
 
@@ -522,30 +470,20 @@ sub validate {
 		  $optional{$k} =  1;
        }
     }
-	
-	# Remove all empty fields
-	foreach my $field ( keys %valid ) {
-		delete $valid{$field} unless length $valid{$field};
-	}
 
-    # Check if the presence of some fields makes other optional fields required.
+    # Remove all empty fields
+    foreach my $field ( keys %valid ) {
+	delete $valid{$field} unless length $valid{$field};
+    }
+
+    # Check if the presence of some fields makes other optional
+    # fields required.
     while ( my ( $field, $deps) = each %{$profile->{dependencies}} ) {
-        if ($valid{$field}) {
-            if (ref($deps) eq 'HASH') {
-                foreach my $key (keys %$deps) {
-                    if($valid{$field} eq $key){
-		        foreach my $dep (_arrayify($deps->{$key})){
-                            $required{$dep} = 1;
-                        }
-                    }
-                }
-            }
-            else {
-                foreach my $dep (_arrayify($deps)){
-                    $required{$dep} = 1;
-                }
-            }
-        }
+	if ( $valid{$field} ) {
+	    foreach my $dep ( _arrayify($deps) ) {
+		$required{$dep} = 1;
+	    }
+	}
     }
 
     # check dependency groups
@@ -579,78 +517,68 @@ sub validate {
     }
 
     # add in the constraints from the regexp map 
-    foreach my $re (keys %{ $profile->{constraint_regexp_map} }) {
-       my $sub = eval 'sub { $_[0] =~ '. $re . '}';
-       die "Error compiling regular expression $re: $@" if $@;
-
-       # find all the keys that match this RE and add a constraint for them
-       map { $profile->{constraints}{$_} = $profile->{constraint_regexp_map}{$re} }
-	 grep { $sub->($_) } (keys %valid);
+    foreach my $k (keys %valid) {
+       foreach my $re (keys %{ $profile->{constraint_regexp_map} }) {
+	  if ($k =~ /$re/) {
+	      $profile->{constraints}{$k} = $profile->{constraint_regexp_map}{$re}; 
+	  }
+       }
     }
 
     # Check constraints
     while ( my ($field,$constraint_spec) = each %{$profile->{constraints}} ) {
-		my ($constraint,@params);
-		if ( ref $constraint_spec eq "HASH" ) {
-			$constraint = $constraint_spec->{constraint};
-			foreach my $fname ( _arrayify($constraint_spec->{params})  ) {
-			push @params, $valid{$fname};
-			}
-		} else {
-			$constraint = $constraint_spec;
-			@params     = ( $valid{$field} );
-		}
-		next unless exists $valid{$field};
-
-		unless ( ref $constraint ) {
-			# Check for regexp constraint
-			if ( $constraint =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ ) {
-				my $sub = eval 'sub { $_[0] =~ '. $constraint . '}';
-				die "Error compiling regular expression $constraint: $@" if $@;
-				$constraint = $sub;
-				# Cache for next use
-				if ( ref $constraint_spec eq "HASH" ) {
-					$constraint_spec->{constraint} = $sub;
-				} else {
-					$profile->{constraints}{$field} = $sub;
-				}
-			} else {
-				# Qualify symbolic reference
-				$constraint = "valid_" . $constraint;
-			}
-		}
-		no strict 'refs';
-
-		unless ( $constraint->( @params ) ) {
-			delete $valid{$field};
-			push @invalid, $field;
-		}
+	my ($constraint,@params);
+	if ( ref $constraint_spec eq "HASH" ) {
+	    $constraint = $constraint_spec->{constraint};
+	    foreach my $fname ( _arrayify($constraint_spec->{params})  ) {
+		push @params, $valid{$fname};
+	    }
+	} else {
+	    $constraint = $constraint_spec;
+	    @params     = ( $valid{$field} );
 	}
+	next unless exists $valid{$field};
 
-    # add back in missing optional fields from the data hash if we need to
-    foreach my $field ( keys %$data ) {
-		if ($profile->{missing_optional_valid} and $optional{$field} and (not exists $valid{$field})) {
-			 $valid{$field} = undef;
-		 }
+	unless ( ref $constraint ) {
+	    # Check for regexp constraint
+	    if ( $constraint =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ ) {
+		my $sub = eval 'sub { $_[0] =~ '. $constraint . '}';
+		die "Error compiling regular expression $constraint: $@" if $@;
+		$constraint = $sub;
+		# Cache for next use
+		if ( ref $constraint_spec eq "HASH" ) {
+		    $constraint_spec->{constraint} = $sub;
+		} else {
+		    $profile->{constraints}{$field} = $sub;
+		}
+	    } else {
+		# Qualify symbolic reference
+		$constraint = "valid_" . $constraint;
+	    }
+	}
+	no strict 'refs';
+
+	unless ( $constraint->( @params ) ) {
+	    delete $valid{$field};
+	    push @invalid, $field;
+	}
     }
-
     return ( \%valid, \@missings, \@invalid, \@unknown );
 }
 
 # takes string or array ref as input
 # returns array
 sub _arrayify {
-   # if the input is undefined, return an empty list
-   my $val = shift;
-   defined $val or return ();
+   # if the input is undefined, we just return that. -mls
+   my $val = shift || return undef;
 
-   if ( ref $val eq 'ARRAY' ) {
+   if ( ref $val ) {
 		# if it's a reference, return an array unless it points an empty array. -mls
-                return (length $val->[0]) ? @$val : ();
+		return $val->[0] ? @$val : undef;   
    } 
    else {
 		# if it's a string, return an array unless the string is missing or empty. -mls
-                return (length $val) ? ($val) : ();
+		return (length $val) ? ($val) : undef;   
    }
 }
 
@@ -1091,7 +1019,7 @@ sub valid_phone {
 =item american_phone
 
 This constraints checks if the number is a possible North American style
-of phone number : (XXX) XXX-XXXX. It has to contains 7 or more digits.
+of phone number : (XXX) XXX-XXXX. It has to contains more than 7 digits.
 
 =cut
 
@@ -1200,52 +1128,6 @@ sub valid_cc_type {
     return $val =~ /^[MVAD]/i;
 }
 
-=pod
-
-=item ip_address
-
-This checks if the input is formatted like an IP address (v4)
-
-=cut
-
-# contributed by Juan Jose Natera Abreu <jnatera@net-uno.net>
-sub valid_ip_address {
-   my $val = shift;
-   if ($val =~ m/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
-     return 
-       (($1 >= 0 && $1 <= 255) && ($2 >= 0 && $2 <= 255) && ($3 >= 0 && $3 <= 255) && ($4 >= 0 && $4 <= 255))
-   }
-}
-
-# check the profile syntax and die if we have an error
-sub _check_profile_syntax {
-	my $profile = shift;
-
-	die "Invalid input profile: needs to be a hash reference\n" unless ref $profile eq "HASH";
-
-	my @valid_profile_keys = (qw/
-		optional
-		required
-		required_regexp 
-		optional_regexp
-		constraints
-		constraint_regexp_map
-		dependencies
-		dependency_groups
-		defaults
-		filters
-		field_filters
-		field_filter_regexp_map
-		missing_optional_valid /);
-
-	# If any of the keys in the profile are not listed as valid keys here, we die with an error	
-	for my $key (keys %$profile) {
-		unless (grep {$key eq $_} @valid_profile_keys) {
-			die "Invalid input profile: $key is not a valid profile key\n"
-		}
-	}
-}
-
 1;
 
 __END__
@@ -1254,11 +1136,6 @@ __END__
 
 =back
 
-=head1 SEE ALSO
-
-L<Data::FormValidator::Tutorial>, perl(1)
-
-
 =head1 CREDITS
 
 Some of those input validation functions have been taken from MiniVend
@@ -1268,11 +1145,10 @@ The credit card checksum validation was taken from contribution by
 Bruce Albrecht <bruce.albrecht@seag.fingerhut.com> to the MiniVend
 program.
 
-=head1 PUBLIC CVS SERVER
+Mark Stosberg contributed a number of enhancements including
+I<required_regexp>, I<optional_regexp> and I<constraint_regexp_map>
 
-Data::FormValidator now has a publicly accessible CVS server provided by
-SourceForge (www.sourceforge.net).  You can access it by going to
-http://sourceforge.net/cvs/?group_id=6582.  You want the module named 'dfv'. 
+ 
 
 =head1 AUTHOR
 
@@ -1283,14 +1159,6 @@ Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
 Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
 Parts Copyright 2001      by Mark Stosberg <mark@summersault.com>
 
-B<Support Mailing List>
- 
-If you have any questions, comments, bug reports or feature suggestions,
-post them to the support mailing list!  To join the mailing list, visit 
-http://lists.sourceforge.net/lists/listinfo/cascade-dataform
-
-=head1 LICENSE 
-
 This program is free software; you can redistribute it and/or modify
 it under the terms as perl itself.
 
